/* The Butterfly Effect
 * This file copyright (C) 2009  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA.
 */

#ifndef TBE_GLOBAL
#define TBE_GLOBAL

#define APPNAME QObject::tr("The Butterfly Effect")
// note: the below CPACK_PACKAGE* variables are passed by cmake,
// see http://bast.fr/talks/cmake/kung-fu/#24
#define APPRELEASE "0.9.3.1"
/* for your special versioning needs, normally empty */
#ifndef APPFLAVOUR
#define APPFLAVOUR ""
#endif
#include <QtGlobal>
#include <QString>

const float PI    = 3.14159265;
const float SQRT2 = 1.41421356;

/* Debug verbosity
 * level 1 = ERROR major errors only
 * level 2 = WARN recoverable errors and warnings
 * level 3 = INFO informational messages
 * level 4 = (default) messages about (unusual) code flow
 * level 5 = DEBUG all debugging, including entry/exit of functions under scrutiny
 * level 6 = debugging info that will affect game timing - should never be enabled
 */
#ifdef QT_DEBUG
#include <QtCore/QtDebug>
#include <ctime>
extern int theVerbosity;	// actually defined in main.cpp
#define MAX_VERBOSITY 6
// note: the below is actually GCC-specific, see "info:/gcc/Variadic Macros"
#define DEBUG1(format, ...)	{if (theVerbosity>=1) qDebug("**** t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
#define DEBUG2(format, ...)	{if (theVerbosity>=2) qDebug("***  t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
#define DEBUG3(format, ...)	{if (theVerbosity>=3) qDebug("**   t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
#define DEBUG4(format, ...)	{if (theVerbosity>=4) qDebug("*    t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
//#define DEBUG4NT(format, ...)	{if (theVerbosity>=4) qDebug(format, ## __VA_ARGS__);}
#define DEBUG5(format, ...)	{if (theVerbosity>=5) qDebug("     t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
#define DEBUG6(format, ...)	{if (theVerbosity>=6) qDebug("     t=%03ld: " format, time(NULL)%1000, ## __VA_ARGS__);}
// only for use during debugging - print a backtrace at current position
extern void printBacktrace(void);
#define DEBUG1ENTRY			{if (theVerbosity>=1) qDebug("**** t=%03ld: entry %s", time(NULL)%1000, Q_FUNC_INFO);}
#define DEBUG3ENTRY			{if (theVerbosity>=3) qDebug("**   t=%03ld: entry %s", time(NULL)%1000, Q_FUNC_INFO);}
#define DEBUG4ENTRY			{if (theVerbosity>=4) qDebug("*    t=%03ld: entry %s", time(NULL)%1000, Q_FUNC_INFO);}
#define DEBUG5ENTRY			{if (theVerbosity>=5) qDebug("     t=%03ld: entry %s", time(NULL)%1000, Q_FUNC_INFO);}
#define DEBUG6ENTRY			{if (theVerbosity>=6) qDebug("     t=%03ld: entry %s", time(NULL)%1000, Q_FUNC_INFO);}
#else
#define MAX_VERBOSITY 0
#define DEBUG1(format, ...)     { /**/ }
#define DEBUG2(format, ...)     { /**/ }
#define DEBUG3(format, ...)     { /**/ }
#define DEBUG4(format, ...)     { /**/ }
#define DEBUG4NT(format, ...)   { /**/ }
#define DEBUG5(format, ...)     { /**/ }
#define DEBUG6(format, ...)     { /**/ }
#define DEBUG1ENTRY			    { /**/ }
#define DEBUG3ENTRY			    { /**/ }
#define DEBUG4ENTRY			    { /**/ }
#define DEBUG5ENTRY			    { /**/ }
#define DEBUG6ENTRY			    { /**/ }
#endif


#define UNUSED_ARG __attribute__((unused))
#define UNUSED_VAR(x) ((void)(x))

/// Converts a QString into a char* .
/// This function is NOT REENTRANT - make sure not to use it twice in the same set of arguments...
/// (implemented in main.cpp)
extern const char* ASCII(const QString& aQString);


// Global variables, defined in main.cpp
#include <atomic>
// technically, all of them need to be atomic because we might be
// accessing/updating them from different threads.
// This prevents potential data races.
extern std::atomic<bool> theDisplayFramerate;
extern std::atomic<bool> theDrawDebug;
extern std::atomic<bool> theDrawPolyOutline;
extern std::atomic<bool> theIsCollisionOn;
extern std::atomic<bool> theIsLevelCreator;
extern std::atomic<bool> theIsRunAsRegression;
extern std::atomic<bool> theTBECaching;
extern QString theStartFileName;

#endif // TBE_GLOBAL
